

;+

function sxt_image_deconvolve,index,data,unc_data,satpix

;NAME:
;     SXT_IMAGE_DECONVOLVE
;PURPOSE:
;     Remove PSF from SXT data
;CATEGORY:
;CALLING SEQUENCE:
;     out = sxt_image_deconvolve(index,data,unc_data,satpix)
;INPUTS:
;     index = SXT index
;     data = SXT data cube
;     unc_data = output from sxt_prep
;     satpix = output from sxt_prep
;OPTIONAL INPUT PARAMETERS:
;KEYWORD PARAMETERS
;OUTPUTS:
;     out = corrected data cube
;COMMON BLOCKS:
;SIDE EFFECTS:
;RESTRICTIONS:
;     The data input must be corrected with sxt_prep first!
;PROCEDURE:
;     Uses image_deconvolve.pro (a simple MEM algorithm)
;MODIFICATION HISTORY:
;     T. Metcalf  2000-10-04
;-

nimage = n_elements(index)
out = float(data)

for iimage = 0L,nimage-1L do begin

   sssat = where(satpix[*,*,iimage])
   if sssat[0] GE 0 then mask = sssat else delvarx,mask

   ; Get PSF
   nx = n_elements(data(*,0,iimage))
   ny = n_elements(data(0,*,iimage))
   res = 2^gt_res(index[iimage])
   xy = double(gt_center(index[iimage])) ; force PSF calculation to be double
   filter = gt_filtb(index[iimage])
   psfsmall = sxt_psf(filter,xy[0],xy[1],dim=64)  ; I assume this is in full res pixels
   ; Rebin PSF to correct resolution
   ; rebin uses neighbor averaging, which is the right thing to do.
   psfsmall = rebin(psfsmall,64/res,64/res)  
   pnx = n_elements(psfsmall(*,0))
   pny = n_elements(psfsmall(0,*))
   nx2 = floor(nx/2.)
   ny2 = floor(ny/2.)
   pnx2 = floor(pnx/2.)
   pny2 = floor(pny/2.)
   psf = dblarr(nx,ny)
   psf[nx2-pnx2,ny2-pny2] = psfsmall   ; Center PSF
   psf = float(psf/total(psf))  ; normalize PSF

   ; Estimate the error
   te00 = alog10(3.e6)  ; Using fixed value makes this approximate
   ph_per_dn = sxt_flux(Te00,index,/photons,/nover) / $
               sxt_flux(Te00,index,gain_ccd=gain_ccd,/nover)
   ;;convert = ph_per_dn*gt_expdur(index,/orig)/gt_expdur(index)
   ;;sigma = sqrt((data*convert)>1.)/convert
   sigma = sxt_dn_unc(index[iimage],data[*,*,iimage],unc_data[*,*,iimage])
   if n_elements(mask) GT 0 then $
      if mask[0] GE 0 then sigma[mask] = 1.0

   ; Do the deconvolution
   print,!stime
   bad = where(data LE 0.0,nbad)
   if nbad GT 0 then $
      message,/info,strcompress('Found ' + string(nbad) + $
                                ' pixels LE 0.  Setting to 0.01')
   ; Use photons rather than DN with usepoisson=1 to get correct Poisson 
   ; statistics in the image.  The convert back to DN.
   new = image_deconvolve((data[*,*,iimage]*ph_per_dn)>.01, $
                          psf,sigma*ph_per_dn,uselog=1,mask=mask, $
                          usepoisson=1,flux=1,usepsfcorr=1)/ph_per_dn
   print,!stime
   out (*,*,iimage) = new

endfor

return,out

end
